@@ -363,7 +363,7 @@ sub Sync {
                         if ( !$SystemGroupsByName{$Group} ) {
                             $Kernel::OM->Get('Kernel::System::Log')->Log(
                                 Priority => 'notice',
-                                Message =>
+                                Message  =>
                                     "Invalid group '$Group' in "
                                     . "'AuthSyncModule::LDAP::UserSyncInitialGroups"
                                     . "$Self->{Count}'!",
@@ -504,7 +504,7 @@ sub Sync {
                 if ( !$SystemGroupsByName{$SyncGroup} ) {
                     $Kernel::OM->Get('Kernel::System::Log')->Log(
                         Priority => 'notice',
-                        Message =>
+                        Message  =>
                             "Invalid group '$SyncGroup' in "
                             . "'AuthSyncModule::LDAP::UserSyncGroupsDefinition"
                             . "$Self->{Count}'!",
@@ -582,7 +582,7 @@ sub Sync {
                             if ( !$SystemGroupsByName{$SyncGroup} ) {
                                 $Kernel::OM->Get('Kernel::System::Log')->Log(
                                     Priority => 'notice',
-                                    Message =>
+                                    Message  =>
                                         "Invalid group '$SyncGroup' in "
                                         . "'AuthSyncModule::LDAP::UserSyncAttributeGroupsDefinition"
                                         . "$Self->{Count}'!",
@@ -644,7 +644,14 @@ sub Sync {
     }
 
     # update changed group permissions
-    if (%GroupPermissionsChanged) {
+    if (
+        %GroupPermissionsChanged
+        && (
+            $UserSyncGroupsDefinition
+            || $UserSyncAttributeGroupsDefinition
+        )
+        )
+    {
         for my $GroupID ( sort keys %GroupPermissionsChanged ) {
 
             $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -720,7 +727,7 @@ sub Sync {
                 if ( !$SystemRolesByName{$SyncRole} ) {
                     $Kernel::OM->Get('Kernel::System::Log')->Log(
                         Priority => 'notice',
-                        Message =>
+                        Message  =>
                             "Invalid role '$SyncRole' in "
                             . "'AuthSyncModule::LDAP::UserSyncRolesDefinition"
                             . "$Self->{Count}'!",
@@ -786,7 +793,7 @@ sub Sync {
                             if ( !$SystemRolesByName{$SyncRole} ) {
                                 $Kernel::OM->Get('Kernel::System::Log')->Log(
                                     Priority => 'notice',
-                                    Message =>
+                                    Message  =>
                                         "Invalid role '$SyncRole' in "
                                         . "'AuthSyncModule::LDAP::UserSyncAttributeRolesDefinition"
                                         . "$Self->{Count}'!",
@@ -806,34 +813,41 @@ sub Sync {
 
     # compare role permissions from ldap with current user role permissions and update if necessary
 
-    # get current user roles
-    my %UserRoles = $GroupObject->PermissionUserRoleGet(
-        UserID => $UserID,
-    );
+    if (
+        $UserSyncRolesDefinition
+        || $UserSyncAttributeRolesDefinition
+        )
+    {
 
-    ROLEID:
-    for my $RoleID ( sort keys %SystemRoles ) {
+        # get current user roles
+        my %UserRoles = $GroupObject->PermissionUserRoleGet(
+            UserID => $UserID,
+        );
 
-        # if old and new permission for role matches, do nothing
-        if (
-            ( $UserRoles{$RoleID} && $RolePermissionsFromLDAP{$RoleID} )
-            ||
-            ( !$UserRoles{$RoleID} && !$RolePermissionsFromLDAP{$RoleID} )
-            )
-        {
-            next ROLEID;
+        ROLEID:
+        for my $RoleID ( sort keys %SystemRoles ) {
+
+            # if old and new permission for role matches, do nothing
+            if (
+                ( $UserRoles{$RoleID} && $RolePermissionsFromLDAP{$RoleID} )
+                ||
+                ( !$UserRoles{$RoleID} && !$RolePermissionsFromLDAP{$RoleID} )
+                )
+            {
+                next ROLEID;
+            }
+
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
+                Priority => 'notice',
+                Message  => "User: '$Param{User}' sync ldap role $SystemRoles{$RoleID}!",
+            );
+            $GroupObject->PermissionRoleUserAdd(
+                UID    => $UserID,
+                RID    => $RoleID,
+                Active => $RolePermissionsFromLDAP{$RoleID} || 0,
+                UserID => 1,
+            );
         }
-
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'notice',
-            Message  => "User: '$Param{User}' sync ldap role $SystemRoles{$RoleID}!",
-        );
-        $GroupObject->PermissionRoleUserAdd(
-            UID    => $UserID,
-            RID    => $RoleID,
-            Active => $RolePermissionsFromLDAP{$RoleID} || 0,
-            UserID => 1,
-        );
     }
 
     # take down session
