@@ -137,6 +137,8 @@ sub ValueSet {
 sub ValueValidate {
     my ( $Self, %Param ) = @_;
 
+    my $LogObject = $Kernel::OM->Get('Kernel::System::Log');
+
     my $Prefix          = 'DynamicField_' . $Param{DynamicFieldConfig}->{Name};
     my $DateRestriction = $Param{DynamicFieldConfig}->{Config}->{DateRestriction};
 
@@ -151,7 +153,7 @@ sub ValueValidate {
         && $Param{Value} !~ m{\A \d{4}-\d{2}-\d{2}\s23:59:59 \z}xms
         )
     {
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
+        $LogObject->Log(
             Priority => 'error',
             Message  => "The value for the Date field ($Param{DynamicFieldConfig}->{Name}) is invalid!\n"
                 . "The date must be valid and the time must be 00:00:00"
@@ -197,17 +199,17 @@ sub ValueValidate {
         }
 
         if ( $DateRestriction eq 'DisableFutureDates' && $ValueSystemTimeObject > $SystemTimeFutureObject ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
+            $LogObject->Log(
                 Priority => 'error',
-                Message =>
+                Message  =>
                     "The value for the Date field ($Param{DynamicFieldConfig}->{Name}) is in the future! The date needs to be in the past!",
             );
             return;
         }
         elsif ( $DateRestriction eq 'DisablePastDates' && $ValueSystemTimeObject < $SystemTimePastObject ) {
-            $Kernel::OM->Get('Kernel::System::Log')->Log(
+            $LogObject->Log(
                 Priority => 'error',
-                Message =>
+                Message  =>
                     "The value for the Date field ($Param{DynamicFieldConfig}->{Name}) is in the past! The date needs to be in the future!",
             );
             return;
@@ -354,38 +356,30 @@ sub EditFieldRender {
         OverrideTimeZone => 1,
     );
 
-    if ( $Param{Mandatory} ) {
-        my $DivID = $FieldName . 'UsedError';
+    # Tooltip for client side validation
+    my $ClientErrorTooltipDivID = $FieldName . 'UsedError';
+    my $FieldRequiredMessage    = $Param{LayoutObject}->{LanguageObject}->Translate("This field is required.");
+    $HTMLString .= <<"EOF";
 
-        my $FieldRequiredMessage = $Param{LayoutObject}->{LanguageObject}->Translate("This field is required.");
-
-        # for client side validation
-        $HTMLString .= <<"EOF";
-
-<div id="$DivID" class="TooltipErrorMessage">
+<div id="$ClientErrorTooltipDivID" class="TooltipErrorMessage">
     <p>
         $FieldRequiredMessage
     </p>
 </div>
 EOF
-    }
 
-    if ( $Param{ServerError} ) {
+    # Tooltip for server side validation
+    my $ErrorMessage = $Param{ErrorMessage} || 'This field is required.';
+    $ErrorMessage = $Param{LayoutObject}->{LanguageObject}->Translate($ErrorMessage);
+    my $ServerErrorTooltipDivID = $FieldName . 'UsedServerError';
+    $HTMLString .= <<"EOF";
 
-        my $ErrorMessage = $Param{ErrorMessage} || 'This field is required.';
-        $ErrorMessage = $Param{LayoutObject}->{LanguageObject}->Translate($ErrorMessage);
-        my $DivID = $FieldName . 'UsedServerError';
-
-        # for server side validation
-        $HTMLString .= <<"EOF";
-
-<div id="$DivID" class="TooltipErrorMessage">
+<div id="$ServerErrorTooltipDivID" class="TooltipErrorMessage">
     <p>
         $ErrorMessage
     </p>
 </div>
 EOF
-    }
 
     # call EditLabelRender on the common Driver
     my $LabelString = $Self->EditLabelRender(
@@ -397,6 +391,7 @@ EOF
         Prefix                => $FieldName,
         $FieldName . Optional => 1,
         $FieldName . Used     => $FieldConfig->{ $FieldName . 'Used' } || 0,
+        $FieldName . 'Class'  => $FieldClass,
     );
 
     my $Data = {
@@ -469,10 +464,10 @@ sub EditFieldValueGet {
             $DynamicFieldValues{ $Prefix . $Type } = sprintf "%02d",
                 $DynamicFieldValues{ $Prefix . $Type };
         }
-        my $Year  = $DynamicFieldValues{ $Prefix . 'Year' }  || '0000';
-        my $Month = $DynamicFieldValues{ $Prefix . 'Month' } || '00';
-        my $Day   = $DynamicFieldValues{ $Prefix . 'Day' }   || '00';
-        my $Hour  = '00';
+        my $Year   = $DynamicFieldValues{ $Prefix . 'Year' }  || '0000';
+        my $Month  = $DynamicFieldValues{ $Prefix . 'Month' } || '00';
+        my $Day    = $DynamicFieldValues{ $Prefix . 'Day' }   || '00';
+        my $Hour   = '00';
         my $Minute = '00';
         my $Second = '00';
 
