@@ -1,11 +1,10 @@
 # --
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
 # did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
-## nofilter(TidyAll::Plugin::OTRS::Perl::Pod::SpellCheck)
 
 package Kernel::System::DynamicField::Webservice;
 
@@ -15,6 +14,7 @@ use warnings;
 our @ObjectDependencies = (
     'Kernel::Config',
     'Kernel::Output::HTML::Layout',
+    'Kernel::System::Cache',
     'Kernel::System::DynamicField',
     'Kernel::System::DynamicField::Backend',
     'Kernel::System::HTMLUtils',
@@ -22,6 +22,7 @@ our @ObjectDependencies = (
     'Kernel::System::Main',
     'Kernel::System::Ticket',
     'Kernel::System::Util',
+    'Kernel::System::Web::Request',
 );
 
 use Kernel::System::VariableCheck qw(:all);
@@ -49,16 +50,17 @@ sub new {
 
     $Self->{RequiredAttributes} = [ 'Webservice', 'InvokerSearch', 'InvokerGet', 'Backend', 'StoredValue', ];
     $Self->{OptionalAttributes} = [
-        'SearchKeys', 'DisplayedValues', 'DisplayedValuesSeparator', 'Limit', 'AutocompleteMinLength', 'QueryDelay',
-        'AdditionalDFStorage', 'InputFieldWidth',
-        'DefaultValue',        'Link',
+        'SearchKeys', 'CacheTTL', 'DisplayedValues', 'DisplayedValuesSeparator', 'Limit', 'AutocompleteMinLength',
+        'QueryDelay', 'AdditionalDFStorage', 'InputFieldWidth', 'DefaultValue',  'Link',
     ];
 
     $Self->{SupportedDynamicFieldTypes} = {
-        WebserviceText        => 1,
+        WebserviceDropdown    => 1,
         WebserviceMultiselect => 1,
     };
 
+    $Self->{CacheType} = 'DynamicFieldWebservice';
+
     return $Self;
 }
 
@@ -79,7 +81,8 @@ Tests given web-service configuration by connecting and querying the web-service
             SearchTerms              => $Param{SearchTerms},
         },
         DynamicFieldName => 'DFName1',      # optional
-        FieldType        => 'WebserviceText',
+        FieldType        => 'WebserviceDropdown',
+        TicketID         => 65, # optional
         UserID           => 1,
         UserType         => 'Agent',                                        # optional 'Agent' or 'Customer'
     );
@@ -132,13 +135,14 @@ sub Test {
 
     my @StoredValue     = split /\s*,\s*/, $DynamicFieldConfig->{StoredValue};
     my @DisplayedValues = split /\s*,\s*/, $DynamicFieldConfig->{DisplayedValues};
-    my %Attributes = map { $_ => 1 } ( 'DisplayValue', @StoredValue, @DisplayedValues );
-    my @Attributes = sort keys %Attributes;
+    my %Attributes      = map { $_ => 1 } ( 'DisplayValue', @StoredValue, @DisplayedValues );
+    my @Attributes      = sort keys %Attributes;
 
     my $Results = $Self->Search(
         DynamicFieldConfig => $DynamicFieldConfig,
         SearchTerms        => $Param{SearchTerms} || $DynamicFieldConfig->{SearchTerms} || '***',
         SearchType         => 'LIKE',
+        TicketID           => $Param{TicketID},                                                     # optional
         UserID             => $Param{UserID},
         Attributes         => \@Attributes,
         UserType           => $Param{UserType},
@@ -171,6 +175,7 @@ Retrieves data for auto-complete list of given dynamic field config.
     my $Results = $DynamicFieldWebserviceObject->Autocomplete(
         DynamicFieldConfig => {},
         SearchTerms        => 'my search',
+        TicketID           => 65, # optional
         UserID             => 1,
         UserType           => 'Agent',                # optional 'Agent' or 'Customer'
     );
@@ -205,6 +210,7 @@ sub Autocomplete {
         SearchTerms        => $Param{SearchTerms},
         SearchKeys         => $Param{SearchKeys},
         SearchType         => 'LIKE',
+        TicketID           => $Param{TicketID},                  # optional
         UserID             => $Param{UserID},
         UserType           => $Param{UserType},
         FieldValues        => $Param{GetParam}->{FieldValues},
@@ -237,6 +243,7 @@ Retrieves data for auto-fill list of given dynamic field config.
     my $Results = $DynamicFieldWebserviceObject->AutoFill(
         DynamicFieldConfig => {},
         SearchTerms        => 'my search',
+        TicketID           => 65, # optional
         UserID             => 1,
         UserType           => 'Agent',                # optional 'Agent' or 'Customer'
     );
@@ -275,6 +282,7 @@ sub AutoFill {
         SearchKeys         => [ $BackendConfig->{StoredValue} ],
         SearchType         => 'EQUALS',
         Attributes         => \@Attributes,
+        TicketID           => $Param{TicketID},                    # optional
         UserID             => $Param{UserID},
         UserType           => $Param{UserType},
     );
@@ -319,7 +327,8 @@ Executes search in configured dynamic field web-service.
             'Name',
             'ID',
         ],
-        UserID => 1,
+        TicketID => 65, # optional
+        UserID   => 1,
     );
 
     $Results is an (empty) array ref of the follwing form or undef on failure.
@@ -341,12 +350,12 @@ Executes search in configured dynamic field web-service.
     my $Results = [
         {
             'Key'  => 'Znuny3',
-            'ID'   => 'Znuny3'
+            'ID'   => 'Znuny3',
             'Name' => 'Znuny3'
         },
         {
             'Key'  => 'Rocks4',
-            'ID'   => 'Rocks4'
+            'ID'   => 'Rocks4',
             'Name' => 'Rocks4'
         }
         # ...
@@ -360,6 +369,7 @@ sub Search {
     my $LogObject    = $Kernel::OM->Get('Kernel::System::Log');
     my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
     my $UtilObject   = $Kernel::OM->Get('Kernel::System::Util');
+    my $CacheObject  = $Kernel::OM->Get('Kernel::System::Cache');
 
     NEEDED:
     for my $Needed (qw(DynamicFieldConfig SearchTerms)) {
@@ -469,6 +479,26 @@ sub Search {
         %Data = ( %Data, %{ $Param{FieldValues} } );
     }
 
+    if ( !$Data{TicketID} ) {
+        my $IsFrontendContext = $UtilObject->IsFrontendContext();
+
+        if ( $Param{TicketID} ) {
+            $Data{TicketID} = $Param{TicketID};
+        }
+        elsif ($IsFrontendContext) {
+
+            # Fallback: Try to get ticket ID from param object.
+            my $ParamObject = $Kernel::OM->Get('Kernel::System::Web::Request');
+            $Data{TicketID} = $ParamObject->GetParam( Param => 'TicketID' );
+
+            # Fallback: Try to get ticket ID from layout object.
+            if ( !$Data{TicketID} ) {
+                my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+                $Data{TicketID} = $LayoutObject->{TicketID};
+            }
+        }
+    }
+
     # Use one request per search term if InvokerGet is being used because
     # some web services don't report the needed data when using InvokerSearch for this.
     if ( $InvokerType eq 'InvokerGet' ) {
@@ -486,11 +516,33 @@ sub Search {
                 UserID      => $Param{UserID},
             );
 
-            my $Results = $BackendObject->Request(
-                %{$BackendConfig},
-                %RequestData,
-                Data => \%Data,
-            );
+            my $CacheKey = $Param{DynamicFieldConfig}->{Name}
+                . '::'
+                . $InvokerType
+                . '::'
+                . $InvokerName
+                . '::'
+                . $SearchTerm
+                . '::'
+                . $Param{UserID};
+
+            my $Results;
+            if ( $BackendConfig->{CacheTTL} ) {
+                $Results = $CacheObject->Get(
+                    Type => $Self->{CacheType},
+                    Key  => $CacheKey,
+                );
+            }
+            my $RequestExecuted;
+            if ( !defined $Results ) {
+                $Results = $BackendObject->Request(
+                    %{$BackendConfig},
+                    %RequestData,
+                    Data => \%Data,
+                );
+
+                $RequestExecuted = 1;
+            }
 
             next SEARCHTERM if !IsArrayRefWithData($Results) && !IsHashRefWithData($Results);
 
@@ -498,6 +550,19 @@ sub Search {
                 $Results = [$Results];
             }
 
+            if (
+                $BackendConfig->{CacheTTL}
+                && $RequestExecuted
+                )
+            {
+                $CacheObject->Set(
+                    Type  => $Self->{CacheType},
+                    Key   => $CacheKey,
+                    Value => $Results,
+                    TTL   => $BackendConfig->{CacheTTL},
+                );
+            }
+
             push @RequestResults, @{$Results};
         }
     }
@@ -511,11 +576,33 @@ sub Search {
             UserID      => $Param{UserID},
         );
 
-        my $Results = $BackendObject->Request(
-            %{$BackendConfig},
-            %RequestData,
-            Data => \%Data,
-        );
+        my $CacheKey = $Param{DynamicFieldConfig}->{Name}
+            . '::'
+            . $InvokerType
+            . '::'
+            . $InvokerName
+            . '::'
+            . $Param{SearchTerms}
+            . '::'
+            . $Param{UserID};
+
+        my $Results;
+        if ( $BackendConfig->{CacheTTL} ) {
+            $Results = $CacheObject->Get(
+                Type => $Self->{CacheType},
+                Key  => $CacheKey,
+            );
+        }
+        my $RequestExecuted;
+        if ( !defined $Results ) {
+            $Results = $BackendObject->Request(
+                %{$BackendConfig},
+                %RequestData,
+                Data => \%Data,
+            );
+
+            $RequestExecuted = 1;
+        }
 
         return if !IsArrayRefWithData($Results) && !IsHashRefWithData($Results);
 
@@ -523,6 +610,19 @@ sub Search {
             $Results = [$Results];
         }
 
+        if (
+            $BackendConfig->{CacheTTL}
+            && $RequestExecuted
+            )
+        {
+            $CacheObject->Set(
+                Type  => $Self->{CacheType},
+                Key   => $CacheKey,
+                Value => $Results,
+                TTL   => $BackendConfig->{CacheTTL},
+            );
+        }
+
         push @RequestResults, @{$Results};
     }
 
@@ -841,7 +941,8 @@ Hashref is used for BuildSelection or input fields.
     my $DisplayValue = $DynamicFieldWebserviceObject->DisplayValueGet(
         DynamicFieldConfig => {},
         Value              => 'My value', # or array of values
-        UserID             => 1,
+        TicketID           => 65, # optional
+        UserID             => 1, # optional
     );
 
     If one value was given, it returns its display value as a string
@@ -915,6 +1016,7 @@ sub DisplayValueGet {
         SearchTerms        => $Param{Value},
         SearchType         => 'EQUALS',
         SearchKeys         => \@SearchKeys,
+        TicketID           => $Param{TicketID},             # optional
         UserID             => $Param{UserID} || 1,
         UserType           => $Param{UserType},
     );
@@ -1040,8 +1142,10 @@ This function is only needed for WebserviceMultiselect (multiselect).
 
     my $Value = $DynamicFieldWebserviceObject->Template(
         DynamicFieldConfig => \%DynamicFieldConfig,
-        Value              => ['first','second', 'third'],
-        ContentType        => 'ASCII', # HTML is default
+        Value              => ['first','second', 'third'], # or a scalar value
+        Type               => 'Value', # optional; or 'Title'
+        ContentType        => 'ASCII', # optional; 'HTML' is default,
+        TemplateType       => 'default', # optional; or: 'separator', 'wordwrap', 'list'
     );
 
 Returns:
@@ -1104,8 +1208,10 @@ sub Template {
     my $TemplateType = $Param{DynamicFieldConfig}->{Config}->{TemplateType} || 'default';
 
     if ( $Param{LayoutObject}->{Action} && $Param{Type} ) {
-        $ContentType  = $ActionTemplateMap{ $Param{LayoutObject}->{Action} }->{ $Param{Type} }->{ContentType};
-        $TemplateType = $ActionTemplateMap{ $Param{LayoutObject}->{Action} }->{ $Param{Type} }->{TemplateType};
+        $ContentType
+            = $ActionTemplateMap{ $Param{LayoutObject}->{Action} }->{ $Param{Type} }->{ContentType} || $ContentType;
+        $TemplateType
+            = $ActionTemplateMap{ $Param{LayoutObject}->{Action} }->{ $Param{Type} }->{TemplateType} || $TemplateType;
     }
 
     if ( $Param{ContentType} ) {
@@ -1304,7 +1410,7 @@ sub _BackendConfigGet {
     if ( !$Self->{SupportedDynamicFieldTypes}->{ $Param{DynamicFieldConfig}->{FieldType} } ) {
         $LogObject->Log(
             Priority => 'error',
-            Message  => "Unsupported dynamic field type $Param{DynamicField}->{FieldType}.",
+            Message  => "Unsupported dynamic field type $Param{DynamicFieldConfig}->{FieldType}.",
         );
         return;
     }
@@ -1327,7 +1433,7 @@ sub _BackendConfigGet {
 
         $LogObject->Log(
             Priority => 'error',
-            Message =>
+            Message  =>
                 "Config option $BackendConfigField is not defined for dynamic field $Param{DynamicFieldConfig}->{Name}.",
         );
         return;
@@ -1336,6 +1442,7 @@ sub _BackendConfigGet {
     BACKENDCONFIGFIELD:
     for my $BackendConfigField ( @{ $Self->{OptionalAttributes} } ) {
         $BackendConfig->{$BackendConfigField} = $Param{DynamicFieldConfig}->{Config}->{$BackendConfigField};
+
         next BACKENDCONFIGFIELD
             if defined $BackendConfig->{$BackendConfigField} && length $BackendConfig->{$BackendConfigField};
 
