@@ -1,6 +1,6 @@
 # --
 # Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -51,8 +51,8 @@ Don't use the constructor directly, use the ObjectManager instead:
     my $PostMasterObject = $Kernel::OM->Create(
         'Kernel::System::PostMaster',
         ObjectParams => {
-            Email        => \@ArrayOfEmailContent,
-            Trusted      => 1, # 1|0 ignore X-OTRS header if false
+            Email   => \@ArrayOfEmailContent,
+            Trusted => 1,                       # 1|0 ignore X-OTRS header if false
         },
     );
 
@@ -209,7 +209,7 @@ sub Run {
             ObjectLogType => 'Message',
             Priority      => 'Info',
             Key           => 'Kernel::System::PostMaster',
-            Value =>
+            Value         =>
                 "Ignored Email (From: $GetParam->{'From'}, Message-ID: $GetParam->{'Message-ID'}) "
                 . "because the X-OTRS-Ignore is set (X-OTRS-Ignore: $GetParam->{'X-OTRS-Ignore'}).",
         );
@@ -547,7 +547,7 @@ sub CheckFollowUp {
                         ObjectLogType => 'Message',
                         Priority      => 'Debug',
                         Key           => 'Kernel::System::PostMaster',
-                        Value =>
+                        Value         =>
                             "Found follow up ticket with TicketNumber '$Ticket{TicketNumber}' and TicketID '$TicketID'.",
                     );
 
@@ -663,7 +663,9 @@ sub GetEmailParams {
     $GetParam{Charset}               = $Self->{ParserObject}->GetReturnCharset();
 
     # Get attachments.
-    my @Attachments = $Self->{ParserObject}->GetAttachments();
+    my @Attachments = $Self->{ParserObject}->GetAttachments(
+        UserType => 'Agent'
+    );
     $GetParam{Attachment} = \@Attachments;
 
     return \%GetParam;
