@@ -1,6 +1,6 @@
 // --
 // Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-// Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+// Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 // --
 // This software comes with ABSOLUTELY NO WARRANTY. For details, see
 // the enclosed file COPYING for license information (GPL). If you
@@ -88,8 +88,9 @@ Core.AJAX = (function (TargetNS) {
         Core.Exception.HandleFinalError(new Core.Exception.ApplicationError(ErrorMessage, 'CommunicationError'));
     }
 
+
     /**
-     * @private
+     * @public
      * @name ToggleAJAXLoader
      * @memberof Core.AJAX
      * @function
@@ -98,7 +99,7 @@ Core.AJAX = (function (TargetNS) {
      * @description
      *      Shows and hides an ajax loader for every element which is updates via ajax.
      */
-    function ToggleAJAXLoader(FieldID, Show) {
+    TargetNS.ToggleAJAXLoader = function (FieldID, Show) {
         var $Element = $('#' + FieldID),
             $Loader = $('#' + AJAXLoaderPrefix + FieldID),
             LoaderHTML = '<span id="' + AJAXLoaderPrefix + FieldID + '" class="AJAXLoader"></span>';
@@ -251,23 +252,26 @@ Core.AJAX = (function (TargetNS) {
             Range,
             StartRange = 0,
             NewPosition = 0,
-            CKEditorObj = parent.CKEDITOR;
+            CKEditorObj;
 
         if ($Element.length) {
             $ParentBody = $Element;
-            ParentBody = $ParentBody[0];
+            ParentBody  = $ParentBody[0];
+            CKEditorObj = Core.UI.RichTextEditor.GetInstance($Element.attr('id'));
 
-            // for regular popups, parent is a reference to the popup itself, which is why parent.CKEDITOR is a reference to the CKEDITOR
+            // TODO: check if needed as CKEditor no longer uses iframe
+            // for regular popups, parent is a reference to the popup itself, which is why parent.CKEditorObj is a reference to the CKEditor
             // object of the popup window. But if we're on a mobile environment, the popup would instead open as an iframe, which would cause
-            // parent.CKEDITOR to be the CKEDITOR object of the parent window which contains the iframe. This is why we want to use only
-            // CKEDITOR in this case (see bug#12680).
-            if (Core.App.Responsive.IsSmallerOrEqual(Core.App.Responsive.GetScreenSize(), 'ScreenL') && (!localStorage.getItem("DesktopMode") || parseInt(localStorage.getItem("DesktopMode"), 10) <= 0)) {
-                CKEditorObj = CKEDITOR;
-            }
+            // parent.CKEditorObj to be the CKEditor object of the parent window which contains the iframe. This is why we want to use only
+            // CKEditor in this case (see bug#12680).
+            // if (Core.App.Responsive.IsSmallerOrEqual(Core.App.Responsive.GetScreenSize(), 'ScreenL') && (!localStorage.getItem("DesktopMode") || parseInt(localStorage.getItem("DesktopMode"), 10) <= 0)) {
+            //     CKEditorObj = CKEditorObj;
+            // }
 
             // add the text to the RichText editor
-            if (CKEditorObj && CKEditorObj.instances.RichText) {
-                CKEditorObj.instances.RichText.focus();
+            if (CKEditorObj) {
+                Core.UI.RichTextEditor.Focus($Element);
+
                 window.setTimeout(function () {
 
                     // In some circumstances, this command throws an error (although inserting the HTML works)
@@ -275,7 +279,7 @@ Core.AJAX = (function (TargetNS) {
                     try {
 
                         // set new text
-                        CKEditorObj.instances.RichText.setData(Value);
+                        CKEditorObj.setData(Value);
                     }
                     catch (Error) {
                         $.noop();
@@ -480,7 +484,7 @@ Core.AJAX = (function (TargetNS) {
 
         if (FieldsToUpdate) {
             $.each(FieldsToUpdate, function (Index, Value) {
-                ToggleAJAXLoader(Value, true);
+                TargetNS.ToggleAJAXLoader(Value, true);
             });
         }
 
@@ -512,7 +516,7 @@ Core.AJAX = (function (TargetNS) {
             complete: function () {
                 if (FieldsToUpdate) {
                     $.each(FieldsToUpdate, function (Index, Value) {
-                        ToggleAJAXLoader(Value, false);
+                        TargetNS.ToggleAJAXLoader(Value, false);
                     });
                 }
             },
@@ -629,5 +633,36 @@ Core.AJAX = (function (TargetNS) {
         });
     };
 
+    TargetNS.FunctionCallSynchronous = function (URL, Data, Callback, DataType) {
+
+        // store the original state
+        // this is basically an example how to access
+        // the current state of the $.ajaxSetup values
+        var OriginalAsyncState = $.ajaxSetup()['async'];
+
+        // make a custom callback that gets passed to the standard Core.AJAX.FunctionCall
+        // that resets back to asynchronous AJAX calls as before and executes the regualar
+        // given Callback function as usual
+        var ResetCallback = function (Response) {
+
+            // set requests back to asynchronous
+            $.ajaxSetup({
+                async: OriginalAsyncState
+            });
+
+            // call given callback function as usual
+            Callback(Response);
+        };
+
+        // set this request as synchronous
+        $.ajaxSetup({
+            async: false
+        });
+
+        // start the wanted request by the framework functionality with our
+        // manipulated callback function and disabled async flag
+        Core.AJAX.FunctionCall(URL, Data, ResetCallback, DataType);
+    };
+
     return TargetNS;
 }(Core.AJAX || {}));
